package com.redxun.knowledge.album.service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.redxun.common.base.db.BaseDao;
import com.redxun.common.base.db.BaseService;
import com.redxun.common.service.impl.SuperServiceImpl;
import com.redxun.common.tool.IdGenerator;
import com.redxun.knowledge.album.entity.consts.AlbumConst;
import com.redxun.knowledge.album.entity.dao.*;
import com.redxun.knowledge.album.entity.dto.YelpDto;
import com.redxun.knowledge.album.mapper.AlbumYelpMapper;
import com.redxun.knowledge.common.UserService;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.List;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.stream.Collectors;

/**
 * [业务--专辑点评表]业务服务类
 */
@Service
public class AlbumYelpServiceImpl extends SuperServiceImpl<AlbumYelpMapper, AlbumYelp> implements BaseService<AlbumYelp> {

    @Resource
    private AlbumYelpMapper albumYelpMapper;

    @Autowired
    private AlbumYelpApprovalNodeServiceImpl albumYelpApprovalNodeService;

    @Autowired
    private AlbumYelpApproveServiceImpl albumYelpApproveService;

    @Autowired
    private UserService userService;

    @Override
    public BaseDao<AlbumYelp> getRepository() {
        return albumYelpMapper;
    }

    /**
     * 根据专辑Id删除点评信息
     *
     * @param ids
     */
    public void deleteByAlbumId(String ids) {
        albumYelpMapper.deleteByAlbumId(ids);
    }

    /**
     * PC端对专辑进行点评或者回复点评（回复）内容
     *
     * @param yelpDto
     * @return
     */
    public String yelp(YelpDto yelpDto) {
        AlbumYelp albumYelp = new AlbumYelp();
        Integer type = yelpDto.getType();
        // 类型为1 点评(1级) 类型为2 回复(2级)
        if (type == 2 && yelpDto.getYelpId() == null){
            return AlbumConst.YELP_FAIL;
        }
        albumYelp.setPkId(IdGenerator.getIdStr());
        BeanUtils.copyProperties(yelpDto, albumYelp);
        albumYelp.setUpdateBy(userService.queryLoginUser());
        albumYelpMapper.insert(albumYelp);
        //推送至审核流程表中
        createAlbumYelpApprove(albumYelp);
        return AlbumConst.YELP_SUCCESS;
    }

    /**
     * 生成最新版审核流程
     *
     * @param albumYelp
     */
    private void createAlbumYelpApprove(AlbumYelp albumYelp) {
        //拉取点评审核节点表
        List<AlbumYelpApprovalNode> albumYelpApprovalNodeList = albumYelpApprovalNodeService.
                findAll(new QueryWrapper<AlbumYelpApprovalNode>().orderByAsc("IS_FINAL", "CREATE_TIME_"));
        //将点评信息推送至点评审核表中(各个节点都推送)
        AtomicInteger sort = new AtomicInteger(1);
        List<AlbumYelpApprove> albumYelpApproveList = albumYelpApprovalNodeList.stream().map(albumYelpApprovalNode -> {
            AlbumYelpApprove albumYelpApprove = new AlbumYelpApprove();
            albumYelpApprove.setPkId(IdGenerator.getIdStr());
            albumYelpApprove.setAlbumId(albumYelp.getAlbumId());

            albumYelpApprove.setApproverId(albumYelpApprovalNode.getApproverId());
            albumYelpApprove.setApproverName(albumYelpApprovalNode.getApproverName());
            albumYelpApprove.setSort(sort.getAndIncrement());
            albumYelpApprove.setIsFinal(albumYelpApprovalNode.getIsFinal());
            albumYelpApprove.setUpdateBy(userService.queryLoginUser());
            albumYelpApprove.setName(albumYelpApprovalNode.getName());
            return albumYelpApprove;
        }).collect(Collectors.toList());
        //批量添加
        albumYelpApproveService.saveBatch(albumYelpApproveList);
    }
}
